# Form implementation generated from reading ui file 'nerdss_install.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_NERDSSInstall(object):
    def setupUi(self, NERDSSInstall):
        NERDSSInstall.setObjectName("NERDSSInstall")
        NERDSSInstall.resize(663, 480)
        self.label = QtWidgets.QLabel(parent=NERDSSInstall)
        self.label.setGeometry(QtCore.QRect(200, 40, 261, 31))
        self.label.setObjectName("label")
        self.lineEditInstallPath = QtWidgets.QLineEdit(parent=NERDSSInstall)
        self.lineEditInstallPath.setGeometry(QtCore.QRect(80, 80, 391, 41))
        self.lineEditInstallPath.setObjectName("lineEditInstallPath")
        self.pushButtonInstallPath = QtWidgets.QPushButton(parent=NERDSSInstall)
        self.pushButtonInstallPath.setGeometry(QtCore.QRect(490, 80, 81, 41))
        self.pushButtonInstallPath.setObjectName("pushButtonInstallPath")
        self.pushButtonCancel = QtWidgets.QPushButton(parent=NERDSSInstall)
        self.pushButtonCancel.setGeometry(QtCore.QRect(170, 180, 80, 25))
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.pushButtonInstall = QtWidgets.QPushButton(parent=NERDSSInstall)
        self.pushButtonInstall.setGeometry(QtCore.QRect(340, 180, 80, 25))
        self.pushButtonInstall.setObjectName("pushButtonInstall")
        self.label_2 = QtWidgets.QLabel(parent=NERDSSInstall)
        self.label_2.setGeometry(QtCore.QRect(80, 250, 511, 61))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtWidgets.QLabel(parent=NERDSSInstall)
        self.label_3.setGeometry(QtCore.QRect(80, 300, 511, 16))
        self.label_3.setObjectName("label_3")

        self.retranslateUi(NERDSSInstall)
        QtCore.QMetaObject.connectSlotsByName(NERDSSInstall)

    def retranslateUi(self, NERDSSInstall):
        _translate = QtCore.QCoreApplication.translate
        NERDSSInstall.setWindowTitle(_translate("NERDSSInstall", "Install NERDSS"))
        self.label.setText(_translate("NERDSSInstall", "Select the Installation Path for NERDSS"))
        self.pushButtonInstallPath.setText(_translate("NERDSSInstall", "Select path"))
        self.pushButtonCancel.setText(_translate("NERDSSInstall", "Cancel"))
        self.pushButtonInstall.setText(_translate("NERDSSInstall", "Install"))
        self.label_2.setText(_translate("NERDSSInstall", "Input your password in the terminal if prompted. The installation might take a while."))
        self.label_3.setText(_translate("NERDSSInstall", "Please wait for the success box to pop up."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    NERDSSInstall = QtWidgets.QDialog()
    ui = Ui_NERDSSInstall()
    ui.setupUi(NERDSSInstall)
    NERDSSInstall.show()
    sys.exit(app.exec())
