# Form implementation generated from reading ui file 'plot_copy_num.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_PlotCopyNum(object):
    def setupUi(self, PlotCopyNum):
        PlotCopyNum.setObjectName("PlotCopyNum")
        PlotCopyNum.resize(640, 480)
        self.listWidgetSpecies = QtWidgets.QListWidget(parent=PlotCopyNum)
        self.listWidgetSpecies.setGeometry(QtCore.QRect(20, 10, 91, 461))
        self.listWidgetSpecies.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.listWidgetSpecies.setObjectName("listWidgetSpecies")
        self.radioButtonPlotSep = QtWidgets.QRadioButton(parent=PlotCopyNum)
        self.radioButtonPlotSep.setGeometry(QtCore.QRect(140, 10, 161, 21))
        self.radioButtonPlotSep.setObjectName("radioButtonPlotSep")
        self.radioButtonPlotSum = QtWidgets.QRadioButton(parent=PlotCopyNum)
        self.radioButtonPlotSum.setGeometry(QtCore.QRect(320, 10, 161, 21))
        self.radioButtonPlotSum.setObjectName("radioButtonPlotSum")
        self.pushButtonPlotSpecies = QtWidgets.QPushButton(parent=PlotCopyNum)
        self.pushButtonPlotSpecies.setGeometry(QtCore.QRect(450, 10, 80, 23))
        self.pushButtonPlotSpecies.setObjectName("pushButtonPlotSpecies")
        self.pushButtonSaveSpecies = QtWidgets.QPushButton(parent=PlotCopyNum)
        self.pushButtonSaveSpecies.setGeometry(QtCore.QRect(310, 450, 131, 21))
        self.pushButtonSaveSpecies.setObjectName("pushButtonSaveSpecies")
        self.widget = MPLWidget(parent=PlotCopyNum)
        self.widget.setGeometry(QtCore.QRect(120, 40, 511, 401))
        self.widget.setObjectName("widget")

        self.retranslateUi(PlotCopyNum)
        QtCore.QMetaObject.connectSlotsByName(PlotCopyNum)

    def retranslateUi(self, PlotCopyNum):
        _translate = QtCore.QCoreApplication.translate
        PlotCopyNum.setWindowTitle(_translate("PlotCopyNum", "Plot Copy Number"))
        self.radioButtonPlotSep.setText(_translate("PlotCopyNum", "Plot Separately"))
        self.radioButtonPlotSum.setText(_translate("PlotCopyNum", "Plot Sum"))
        self.pushButtonPlotSpecies.setText(_translate("PlotCopyNum", "Plot"))
        self.pushButtonSaveSpecies.setText(_translate("PlotCopyNum", "Save to CSV"))
from .mplwidget import MPLWidget


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    PlotCopyNum = QtWidgets.QWidget()
    ui = Ui_PlotCopyNum()
    ui.setupUi(PlotCopyNum)
    PlotCopyNum.show()
    sys.exit(app.exec())
